@@ -43,8 +43,8 @@
 // guarantees at all.
 //
 #define OIIO_VERSION_MAJOR 2
-#define OIIO_VERSION_MINOR 3
-#define OIIO_VERSION_PATCH 14
+#define OIIO_VERSION_MINOR 4
+#define OIIO_VERSION_PATCH 6
 #define OIIO_VERSION_TWEAK 0
 #define OIIO_VERSION_RELEASE_TYPE 
 
@@ -73,24 +73,30 @@
 // We also define the old name for backwards compatibility purposes.
 #define OPENIMAGEIO_VERSION OIIO_VERSION
 
-// Magic macros to make OIIO_VERSION_STRING that looks like "1.2.3"
-#define OIIO_MAKE_VERSION_STRING2(a,b,c,d) #a "." #b "." #c #d
-#define OIIO_MAKE_VERSION_STRING(a,b,c,d) OIIO_MAKE_VERSION_STRING2(a,b,c,d)
+// Magic macros to make OIIO_VERSION_STRING that looks like "1.2.3.0"
+#define OIIO_MAKE_VERSION_STRING2(a,b,c,d,e) #a "." #b "." #c "." #d #e
+#define OIIO_MAKE_VERSION_STRING(a,b,c,d,e) OIIO_MAKE_VERSION_STRING2(a,b,c,d,e)
 #define OIIO_VERSION_STRING \
     OIIO_MAKE_VERSION_STRING(OIIO_VERSION_MAJOR, \
                              OIIO_VERSION_MINOR, OIIO_VERSION_PATCH, \
-                             OIIO_VERSION_RELEASE_TYPE)
+                             OIIO_VERSION_TWEAK, OIIO_VERSION_RELEASE_TYPE)
 #define OIIO_INTRO_STRING "OpenImageIO " OIIO_VERSION_STRING " http://www.openimageio.org"
 
+// Only major.minor.patch.tweak, omit any release type
+#define OIIO_VERSION_STRING_MMPT \
+    OIIO_MAKE_VERSION_STRING(OIIO_VERSION_MAJOR, \
+                             OIIO_VERSION_MINOR, OIIO_VERSION_PATCH, \
+                             OIIO_VERSION_TWEAK, "")
+
 
 // Establish the name spaces
-namespace OpenImageIO_v2_3 { }
-namespace OIIO = OpenImageIO_v2_3;
+namespace OpenImageIO_v2_4 { }
+namespace OIIO = OpenImageIO_v2_4;
 
 // Macros to use in each file to enter and exit the right name spaces.
-#define OIIO_NAMESPACE OpenImageIO_v2_3
-#define OIIO_NAMESPACE_STRING "OpenImageIO_v2_3"
-#define OIIO_NAMESPACE_BEGIN namespace OpenImageIO_v2_3 {
+#define OIIO_NAMESPACE OpenImageIO_v2_4
+#define OIIO_NAMESPACE_STRING "OpenImageIO_v2_4"
+#define OIIO_NAMESPACE_BEGIN namespace OpenImageIO_v2_4 {
 #define OIIO_NAMESPACE_END }
 #define OIIO_NAMESPACE_USING using namespace OIIO;
 
@@ -148,6 +154,10 @@ namespace OIIO = OpenImageIO_v2_3;
 #define OIIO_PLUGIN_EXPORTS_END }
 #endif
 
+// Which version of Imath is used in the OIIO API?
+#define OIIO_USING_IMATH_VERSION_MAJOR 3
+#define OIIO_USING_IMATH_VERSION_MINOR 1
+
 // Which CPP standard (11, 14, etc.) was this copy of OIIO *built* with?
 #define OIIO_BUILD_CPP 14
 
@@ -157,5 +167,9 @@ namespace OIIO = OpenImageIO_v2_3;
 #define OIIO_BUILD_CPP17 (14 >= 17)
 #define OIIO_BUILD_CPP20 (14 >= 20)
 
-#endif
 
+// Was the project built with TBB support?
+#define OIIO_TBB 0
+
+
+#endif  /* defined(OPENIMAGEIO_VERSION_H) */
