@@ -1,6 +1,6 @@
 // Copyright 2008-present Contributors to the OpenImageIO project.
 // SPDX-License-Identifier: BSD-3-Clause
-// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
+// https://github.com/OpenImageIO/oiio
 
 
 /////////////////////////////////////////////////////////////////////////
@@ -19,6 +19,7 @@
 
 #pragma once
 
+#include <cassert>
 #include <cstdlib>
 #include <type_traits>
 #include <utility>  // std::forward
@@ -47,6 +48,9 @@
 #    ifndef NOMINMAX
 #        define NOMINMAX
 #    endif
+#    ifndef NOGDI
+#        define NOGDI
+#    endif
 #    include <windows.h>
 #endif
 
@@ -54,51 +58,56 @@
 #    include <intrin.h>
 #endif
 
+//Avoid min and max being defined
+#if defined(_WIN32)
+#define NOMINMAX
+#endif
+
 #include <OpenImageIO/oiioversion.h>
 #include <OpenImageIO/export.h>
 
 // Detect which C++ standard we're using, and handy macros.
 // See https://en.cppreference.com/w/cpp/compiler_support
 //
-// OIIO_CPLUSPLUS_VERSION : which C++ standard is compiling (11, 14, ...)
-// OIIO_CONSTEXPR14 : constexpr for C++ >= 14, otherwise nothing (this is
-//                      useful for things that can only be constexpr for 14)
-// OIIO_CONSTEXPR17 : constexpr for C++ >= 17, otherwise nothing (this is
-//                      useful for things that can only be constexpr for 17)
+// OIIO_CPLUSPLUS_VERSION : which C++ standard is compiling (14, 17, ...)
+// OIIO_CONSTEXPR14 :
+// OIIO_CONSTEXPR17 :
+// OIIO_CONSTEXPR20 : constexpr for C++ >= the designated version, otherwise
+//                    nothing (this is useful for things that can only be
+//                    constexpr for particular versions or greater).
+// OIIO_INLINE_CONSTEXPR : inline constexpr variables, added in C++17. For
+//                         older C++, static constexpr.
 //
-// Note: oiioversion.h defines OIIO_BUILD_CPP11, OIIO_BUILD_CPP14,
-// OIIO_BUILD_CPP17, or OIIO_BUILD_CPP20 to be 1 if OIIO itself was *built*
-// using C++11, C++14, C++17, or C++20, respectively. In contrast,
+// Note: oiioversion.h defines OIIO_BUILD_CPP (set to 14, 17, etc.)
+// reflecting what OIIO itself was *built* with.  In contrast,
 // OIIO_CPLUSPLUS_VERSION defined below will be set to the right number for
 // the C++ standard being compiled RIGHT NOW. These two things may be the
 // same when compiling OIIO, but they may not be the same if another
 // package is compiling against OIIO and using these headers (OIIO may be
-// C++11 but the client package may be newer, or vice versa -- use these two
+// C++14 but the client package may be newer, or vice versa -- use these two
 // symbols to differentiate these cases, when important).
 #if (__cplusplus >= 202001L)
 #    define OIIO_CPLUSPLUS_VERSION 20
-#    define OIIO_CONSTEXPR14 constexpr
 #    define OIIO_CONSTEXPR17 constexpr
 #    define OIIO_CONSTEXPR20 constexpr
+#    define OIIO_INLINE_CONSTEXPR inline constexpr
 #elif (__cplusplus >= 201703L)
 #    define OIIO_CPLUSPLUS_VERSION 17
-#    define OIIO_CONSTEXPR14 constexpr
 #    define OIIO_CONSTEXPR17 constexpr
 #    define OIIO_CONSTEXPR20 /* not constexpr before C++20 */
+#    define OIIO_INLINE_CONSTEXPR inline constexpr
 #elif (__cplusplus >= 201402L) || (defined(_MSC_VER) && _MSC_VER >= 1914)
 #    define OIIO_CPLUSPLUS_VERSION 14
-#    define OIIO_CONSTEXPR14 constexpr
-#    define OIIO_CONSTEXPR17 /* not constexpr before C++17 */
-#    define OIIO_CONSTEXPR20 /* not constexpr before C++20 */
-#elif (__cplusplus >= 201103L) || (defined(_MSC_VER) && _MSC_VER >= 1900)
-#    define OIIO_CPLUSPLUS_VERSION 11
-#    define OIIO_CONSTEXPR14 /* not constexpr before C++14 */
 #    define OIIO_CONSTEXPR17 /* not constexpr before C++17 */
 #    define OIIO_CONSTEXPR20 /* not constexpr before C++20 */
+#    define OIIO_INLINE_CONSTEXPR static constexpr
 #else
-#    error "This version of OIIO is meant to work only with C++11 and above"
+#    error "This version of OIIO is meant to work only with C++14 and above"
 #endif
 
+// DEPRECATED(2.3): use C++14 constexpr
+#define OIIO_CONSTEXPR14 constexpr
+
 // DEPRECATED(1.8): use C++11 constexpr
 #define OIIO_CONSTEXPR constexpr
 #define OIIO_CONSTEXPR_OR_CONST constexpr
@@ -133,9 +142,14 @@
 //   __clang__ is defined for all clang varieties (generic and Apple)
 //   __apple_build_version__ is only defined for Apple clang
 //   __INTEL_COMPILER is defined only for icc
+//   __INTEL_LLVM_COMPILER is defined only for icx
 //   _MSC_VER is defined for MSVS compiler (not gcc/clang/icc even on Windows)
 //   _WIN32 is defined on Windows regardless of compiler
-//   __CUDACC__ is defined for nvcc and clang during Cuda compilation.
+//   __CUDACC__   is defined any time we are compiling a module for Cuda
+//                (both for the host pass and the device pass). "Do this
+//                when using nvcc or clang with ptx target."
+//   __CUDA_ARCH__  is only defined when doing the device pass. "Do this only
+//                for code that will actually run on the GPU."
 
 
 // Define OIIO_GNUC_VERSION to hold an encoded gcc version (e.g. 40802 for
@@ -149,8 +163,8 @@
 // Define OIIO_CLANG_VERSION to hold an encoded generic Clang version (e.g.
 // 30402 for clang 3.4.2), or 0 if not a generic Clang release.
 // N.B. This will be 0 for the clang Apple distributes (which has different
-// version numbers entirely).
-#if defined(__clang__) && !defined(__apple_build_version__)
+// version numbers entirely) and for the Intel clang-based compiler.
+#if defined(__clang__) && !defined(__apple_build_version__) && !defined(__INTEL_LLVM_COMPILER)
 #  define OIIO_CLANG_VERSION (10000*__clang_major__ + 100*__clang_minor__ + __clang_patchlevel__)
 #else
 #  define OIIO_CLANG_VERSION 0
@@ -159,39 +173,68 @@
 // Define OIIO_APPLE_CLANG_VERSION to hold an encoded Apple Clang version
 // (e.g. 70002 for clang 7.0.2), or 0 if not an Apple Clang release.
 #if defined(__clang__) && defined(__apple_build_version__)
+#  if defined(__INTEL_LLVM_COMPILER)
+#    error Not expected for __INTEL_LLVM_COMPILER to be defined with an __apple_build_version__
+     // The classic Intel(r) C++ Compiler on OSX may still define __clang__.
+     // Combine with testing OIIO_INTEL_COMPILER to further differentiate if
+     // needed.
+#  endif
 #  define OIIO_APPLE_CLANG_VERSION (10000*__clang_major__ + 100*__clang_minor__ + __clang_patchlevel__)
 #else
 #  define OIIO_APPLE_CLANG_VERSION 0
 #endif
 
-// Define OIIO_INTEL_COMPILER_VERSION to hold an encoded Intel compiler
-// version (e.g. 1900), or 0 if not an Intel compiler.
+// Define OIIO_INTEL_CLASSIC_COMPILER_VERSION to hold an encoded Intel
+// compiler version (e.g. 1900), or 0 if not an Intel compiler.
 #if defined(__INTEL_COMPILER)
-#  define OIIO_INTEL_COMPILER_VERSION __INTEL_COMPILER
+#  define OIIO_INTEL_CLASSIC_COMPILER_VERSION __INTEL_COMPILER
 #else
-#  define OIIO_INTEL_COMPILER_VERSION 0
+#  define OIIO_INTEL_CLASSIC_COMPILER_VERSION 0
 #endif
 
-// Intel's compiler on OSX may still define __clang__ and we have need to
-// know when using a true clang compiler.
+// DEPRECATED(2.4) phase out OIIO_NON_INTEL_CLANG for OIIO_INTEL_LLVM_COMPILER.
+// We're keeping the old one for a while for back compatibility.
 #if !defined(__INTEL_COMPILER) && defined(__clang__)
 #  define OIIO_NON_INTEL_CLANG  __clang__
 #else
 #  define OIIO_NON_INTEL_CLANG  0
 #endif
 
+// Define OIIO_INTEL_LLVM_COMPILER to hold an encoded Intel(r) LLVM Compiler
+// version (e.g. 20220000), or 0 if not an Intel(r) LLVM Compiler.
+// Define OIIO_INTEL_CLANG_VERSION to hold the encoded Clang version the
+// Intel(r) LLVM Compiler is based on (e.g. 140000), or 0 if not an Intel(r)
+// LLVM compiler.
+#if defined(__INTEL_LLVM_COMPILER)
+#  define OIIO_INTEL_LLVM_COMPILER __INTEL_LLVM_COMPILER
+#  define OIIO_INTEL_CLANG_VERSION (10000*__clang_major__ + 100*__clang_minor__ + __clang_patchlevel__)
+#else
+#  define OIIO_INTEL_LLVM_COMPILER 0
+#  define OIIO_INTEL_CLANG_VERSION 0
+#endif
+
+// Define OIIO_ANY_CLANG to 0 or 1 to indicate if any Clang based compiler is
+// in use.
+#if defined(__clang__)
+#  define OIIO_ANY_CLANG 1
+#else
+#  define OIIO_ANY_CLANG 0
+#endif
+
 // Tests for MSVS versions, always 0 if not MSVS at all.
 #if defined(_MSC_VER)
-#  if _MSC_VER < 1900
-#    error "This version of OIIO is meant to work only with Visual Studio 2015 or later"
-#  endif
+#  define OIIO_MSVS_VERSION       _MSC_VER
 #  define OIIO_MSVS_AT_LEAST_2013 (_MSC_VER >= 1800)
 #  define OIIO_MSVS_BEFORE_2013   (_MSC_VER <  1800)
 #  define OIIO_MSVS_AT_LEAST_2015 (_MSC_VER >= 1900)
 #  define OIIO_MSVS_BEFORE_2015   (_MSC_VER <  1900)
 #  define OIIO_MSVS_AT_LEAST_2017 (_MSC_VER >= 1910)
 #  define OIIO_MSVS_BEFORE_2017   (_MSC_VER <  1910)
+#  if OIIO_MSVS_BEFORE_2017
+#    error "This version of OIIO is meant to work only with Visual Studio 2017 or later"
+#  endif
 #else
+#  define OIIO_MSVS_VERSION       0
 #  define OIIO_MSVS_AT_LEAST_2013 0
 #  define OIIO_MSVS_BEFORE_2013   0
 #  define OIIO_MSVS_AT_LEAST_2015 0
@@ -209,6 +252,8 @@
 // OIIO_GCC_PRAGMA(x) -- pragma on gcc/clang/icc only
 // OIIO_CLANG_PRAGMA(x) -- pragma on clang only (not gcc or icc)
 // OIIO_MSVS_PRAGMA(x) -- pragma on MSVS only
+// OIIO_INTEL_PRAGMA(x) -- pragma on Intel icc compiler only
+// OIIO_INTEL_LLVM_PRAGMA(x) -- pragma on Intel icx compiler only
 
 // Generic pragma definition
 #if defined(_MSC_VER)
@@ -219,7 +264,7 @@
     #define OIIO_PRAGMA(UnQuotedPragma) _Pragma(#UnQuotedPragma)
 #endif
 
-#if defined(__GNUC__) /* gcc, clang, icc */
+#if defined(__GNUC__) /* gcc, clang, icc, icx */
 #    define OIIO_PRAGMA_WARNING_PUSH    OIIO_PRAGMA(GCC diagnostic push)
 #    define OIIO_PRAGMA_WARNING_POP     OIIO_PRAGMA(GCC diagnostic pop)
 #    define OIIO_PRAGMA_VISIBILITY_PUSH OIIO_PRAGMA(GCC visibility push(default))
@@ -232,6 +277,16 @@
 #        define OIIO_CLANG_PRAGMA(UnQuotedPragma)
 #        define OIIO_GCC_ONLY_PRAGMA(UnQuotedPragma) OIIO_PRAGMA(UnQuotedPragma)
 #    endif
+#    if defined(__INTEL_COMPILER)
+#        define OIIO_INTEL_PRAGMA(UnQuotedPragma) OIIO_PRAGMA(UnQuotedPragma)
+#    else
+#        define OIIO_INTEL_PRAGMA(UnQuotedPragma)
+#    endif
+#    if defined(__INTEL_LLVM_COMPILER)
+#        define OIIO_INTEL_LLVM_PRAGMA(UnQuotedPragma) OIIO_PRAGMA(UnQuotedPragma)
+#    else
+#        define OIIO_INTEL_LLVM_PRAGMA(UnQuotedPragma)
+#    endif
 #    define OIIO_MSVS_PRAGMA(UnQuotedPragma)
 #elif defined(_MSC_VER)
 #    define OIIO_PRAGMA_WARNING_PUSH __pragma(warning(push))
@@ -241,6 +296,8 @@
 #    define OIIO_GCC_PRAGMA(UnQuotedPragma)
 #    define OIIO_GCC_ONLY_PRAGMA(UnQuotedPragma)
 #    define OIIO_CLANG_PRAGMA(UnQuotedPragma)
+#    define OIIO_INTEL_PRAGMA(UnQuotedPragma)
+#    define OIIO_INTEL_LLVM_PRAGMA(UnQuotedPragma)
 #    define OIIO_MSVS_PRAGMA(UnQuotedPragma) OIIO_PRAGMA(UnQuotedPragma)
 #else
 #    define OIIO_PRAGMA_WARNING_PUSH
@@ -250,22 +307,53 @@
 #    define OIIO_GCC_PRAGMA(UnQuotedPragma)
 #    define OIIO_GCC_ONLY_PRAGMA(UnQuotedPragma)
 #    define OIIO_CLANG_PRAGMA(UnQuotedPragma)
+#    define OIIO_INTEL_PRAGMA(UnQuotedPragma)
+#    define OIIO_INTEL_LLVM_PRAGMA(UnQuotedPragma)
 #    define OIIO_MSVS_PRAGMA(UnQuotedPragma)
 #endif
 
 
 
-/// allocates smallish stack memory, equivalent of C99 type var_name[size]
+/// OIIO_ALLOCA is used to allocate smallish amount of memory on the stack,
+/// equivalent of C99 type var_name[size].
+///
+/// NOTE: in a debug build, this will assert for allocations >= 1MB, which
+/// is much too big. Hopefully this will keep us from abusing alloca and
+/// having stack overflows. The rule of thumb is that it's ok to use alloca
+/// for small things of bounded size (like, one float per channel), but
+/// not for anything that could be arbitrarily big (like a full scanline or
+/// image, because sooner or later somebody will give you an image big
+/// enough to cause trouble). Consider using the OIIO_ALLOCATE_STACK_OR_HEAP
+/// idiom rather than a direct OIIO_ALLOCA if you aren't sure the item will
+/// be small.
 #if defined(__GNUC__)
-#    define OIIO_ALLOCA(type, size) ((size) != 0 ? ((type*)__builtin_alloca((size) * sizeof(type))) : nullptr)
+#    define OIIO_ALLOCA(type, size) (assert(size < (1<<20)), (size) != 0 ? ((type*)__builtin_alloca((size) * sizeof(type))) : nullptr)
 #else
-#    define OIIO_ALLOCA(type, size) ((size) != 0 ? ((type*)alloca((size) * sizeof(type))) : nullptr)
+#    define OIIO_ALLOCA(type, size) (assert(size < (1<<20)), (size) != 0 ? ((type*)alloca((size) * sizeof(type))) : nullptr)
 #endif
 
 /// Deprecated (for namespace pollution reasons)
 #define ALLOCA(type, size) OIIO_ALLOCA(type, size)
 
 
+/// Try to allocate T* var to point to T[size] elements of temporary storage
+/// that will automatically free when the local scope is exited. Allocate
+/// the space on the stack with alloca if it's small, but if it's big (> 64
+/// KB), allocate on the heap with a new[], stored as a std::unique_ptr. In
+/// both cases, the memory will be freed automatically upon exit of scope.
+/// That threshold is big enough for one scanline of a 4096 x 4 channel x
+/// float image, or one 64x64 tile of a 4xfloat image.
+#define OIIO_ALLOCATE_STACK_OR_HEAP(var, T, size)   \
+    size_t var##___size = size_t(size);             \
+    std::unique_ptr<T[]> var##___heap;              \
+    if (var##___size * sizeof(T) <= (1 << 16)) {    \
+        var = OIIO_ALLOCA(T, var##___size);         \
+    } else {                                        \
+        var##___heap.reset(new T[var##___size]);    \
+        var = var##___heap.get();                   \
+    }
+
+
 // Define a macro that can be used for memory alignment.
 // This macro is mostly obsolete and C++11 alignas() should be preferred
 // for new code.
@@ -323,7 +411,7 @@
 
 // OIIO_PURE_FUNC is a function attribute that assures the compiler that the
 // function does not write to any non-local memory other than its return
-// value and has no side effects. This can ebable additional compiler
+// value and has no side effects. This can enable additional compiler
 // optimizations by knowing that calling the function cannot possibly alter
 // any other memory. This declaration goes after the function declaration:
 //   int blah (int arg) OIIO_PURE_FUNC;
@@ -409,13 +497,25 @@
 // false positives that you can't easily get rid of.
 // This should work for any clang >= 3.3 and gcc >= 4.8, which are
 // guaranteed by our minimum requirements.
-#if defined(__clang__) || defined(__GNUC__) || __has_attribute(no_sanitize_address)
+#if defined(__clang__) || (OIIO_GNUC_VERSION > 90000 && !defined(__INTEL_COMPILER)) \
+                       || __has_attribute(no_sanitize_address)
 #    define OIIO_NO_SANITIZE_ADDRESS __attribute__((no_sanitize_address))
 #else
 #    define OIIO_NO_SANITIZE_ADDRESS
 #endif
 
 
+// OIIO_NO_SANITIZE_UNDEFINED can be used to mark a function that you don't
+// want undefined behavior sanitizer to catch. Only use this if you know there
+// are false positives that you can't easily get rid of.
+#if defined(__clang__) || (OIIO_GNUC_VERSION > 90000 && !defined(__INTEL_COMPILER)) \
+                       || __has_attribute(no_sanitize)
+#    define OIIO_NO_SANITIZE_UNDEFINED __attribute__((no_sanitize("undefined")))
+#else
+#    define OIIO_NO_SANITIZE_UNDEFINED
+#endif
+
+
 // OIIO_RETURNS_NONNULL following a function declaration of a function
 // indicates that the pointer returned by the function is guaranteed to
 // never be nullptr.
@@ -489,7 +589,7 @@ inline void cpuid (int info[4], int infoType, int extra)
 {
     // Implementation cribbed from Halide (http://halide-lang.org), which
     // cribbed it from ISPC (https://github.com/ispc/ispc).
-#if (defined(_WIN32) || defined(__i386__) || defined(__x86_64__))
+#if (defined(_M_X64) || defined(_M_IX86) || defined(_M_AMD64) || defined(__i386__) || defined(__x86_64__))
 # ifdef _MSC_VER
     __cpuidex(info, infoType, extra);
 # elif defined(__x86_64__)
@@ -505,6 +605,9 @@ inline void cpuid (int info[4], int infoType, int extra)
         : "=a" (info[0]), "=r" (info[1]), "=c" (info[2]), "=d" (info[3])
         : "0" (infoType), "2" (extra));
 # endif
+#elif (defined(_M_ARM64) || defined (__aarch64__) || defined(__aarch64))
+    info[0] = 0; info[1] = 0; info[2] = 0; info[3] = 0;
+    #define __ARM_NEON__
 #else
     info[0] = 0; info[1] = 0; info[2] = 0; info[3] = 0;
 #endif
@@ -540,7 +643,10 @@ template <typename T, class... Args>
 inline T* aligned_new(Args&&... args) {
     static_assert(alignof(T) > alignof(void*), "Type doesn't seem to be over-aligned, aligned_new is not required");
     void* ptr = aligned_malloc(sizeof(T), alignof(T));
+    OIIO_PRAGMA_WARNING_PUSH
+    OIIO_INTEL_PRAGMA(warning disable 873)
     return ptr ? new (ptr) T(std::forward<Args>(args)...) : nullptr;
+    OIIO_PRAGMA_WARNING_POP
 }
 
 template <typename T>
@@ -564,7 +670,8 @@ inline void aligned_delete(T* t) {
 // An enable_if helper to be used in template parameters which results in
 // much shorter symbols: https://godbolt.org/z/sWw4vP
 // Borrowed from fmtlib.
-#define OIIO_ENABLE_IF(...) OIIO::enable_if_t<(__VA_ARGS__), int> = 0
-
+#ifndef OIIO_ENABLE_IF
+#   define OIIO_ENABLE_IF(...) OIIO::enable_if_t<(__VA_ARGS__), int> = 0
+#endif
 
 OIIO_NAMESPACE_END
